// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate;

import java.util.List;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import javax.swing.*;

import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.NonNls;
import org.jdom.Element;

import com.intellij.openapi.components.ApplicationComponent;
import com.intellij.openapi.options.Configurable;
import com.intellij.openapi.options.ConfigurationException;
import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.extensions.PluginId;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.util.JDOMExternalizable;
import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.util.WriteExternalException;

import com.kiwisoft.generate.model.*;
import com.kiwisoft.generate.formats.*;
import com.kiwisoft.generate.lookups.TypeLookup;
import com.kiwisoft.utils.gui.table.TableEditorFactory;
import com.kiwisoft.utils.gui.table.TableLookupEditor;
import com.kiwisoft.utils.gui.Icons;
import com.kiwisoft.utils.format.FormatManager;
import com.kiwisoft.utils.Utils;
import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.VersionInfo;
import com.kiwisoft.utils.idea.PluginUtils;

/**
 * The plugins main component.
 *
 * @author Stefan Stiller
 */
public class GeneratorPlugin implements ApplicationComponent, Configurable, JDOMExternalizable
{
	public static GeneratorPlugin getInstance()
	{
		return ApplicationManager.getApplication().getComponent(GeneratorPlugin.class);
	}

	private static VersionInfo versionInfo=new VersionInfo("com.kiwisoft.generate.Build");

	public static VersionInfo getVersionInfo()
	{
		return versionInfo;
	}

	public GeneratorPlugin()
	{
	}

	public void initComponent()
	{
		FormatManager formatManager=FormatManager.getInstance();
		formatManager.setFormat(Template.class, new TemplateFormat());
		formatManager.setFormat(Parameter.class, new ParameterFormat());
		formatManager.setFormat(Type.class, new TypeFormat());
		formatManager.setFormat(TemplateFile.class, new TemplateFileFormat());
		formatManager.setFormat(SystemMethod.class, new MethodFormat());

		TableEditorFactory editorFactory=TableEditorFactory.getInstance();
		editorFactory.setEditor(Type.class, new TableLookupEditor(new TypeLookup()));

		initializeActions();
	}

	public static void initializeActions()
	{
		ActionManager actionManager=ActionManager.getInstance();
		DefaultActionGroup actionGroup=(DefaultActionGroup)actionManager.getAction("NewGroup");
		for (String actionId : actionManager.getActionIds("KiwiCreate"))
		{
			AnAction action=actionManager.getAction(actionId);
			actionGroup.remove(action);
			actionManager.unregisterAction(actionId);
		}
		String previousActionId="NewDir";
		for (Template template : Utils.sortedSet(TemplateManager.getInstance().getTemplates()))
		{
			AnAction action=new GenerateAction(template);
			String actionId="KiwiCreate.Template."+template.getName();
			actionManager.registerAction(actionId, action, PluginId.getId("KiwiCreate"));
			actionGroup.add(action, new Constraints(Anchor.AFTER, previousActionId));
			previousActionId=actionId;
		}
	}

	public void disposeComponent()
	{
	}

	public String getComponentName()
	{
		return "KiwiGenerator";
	}

	// Configurable interface methods

	public String getDisplayName()
	{
		return "Kiwi Generator";
	}

	public Icon getIcon()
	{
		return Icons.getIcon("application");
	}

	@Nullable
	@NonNls
	public String getHelpTopic()
	{
		return "KiwiGenerator.templates.overview";
	}

	private ConfigurationPanel panel;

	public JComponent createComponent()
	{
		panel=new ConfigurationPanel();
		return panel;
	}

	public boolean isModified()
	{
		return panel.isModified();
	}

	public void apply() throws ConfigurationException
	{
		panel.apply();
	}

	public void reset()
	{
		panel.reset();
	}

	public void disposeUIResources()
	{
		if (panel!=null) panel.dispose();
		panel=null;
	}

	public void readExternal(Element element) throws InvalidDataException
	{
		List children=element.getChildren("parameter");
		if (children!=null)
		{
			Map<String, String> parameters=new HashMap<String, String>();
			for (Iterator it=children.iterator(); it.hasNext();)
			{
				Element child=(Element)it.next();
				String name=PluginUtils.getString(child, "name", null);
				if (!StringUtils.isEmpty(name))
				{
					String value=PluginUtils.getString(child, "value", null);
					parameters.put(name, value);
				}
			}
			TemplateManager.getInstance().setParameters(parameters);
		}
	}

	public void writeExternal(Element element) throws WriteExternalException
	{
		Map<String, String> parameters=TemplateManager.getInstance().getParameters();
		if (parameters!=null)
		{
			for (Map.Entry<String, String> parameter : parameters.entrySet())
			{
				Element child=new Element("parameter");
				PluginUtils.setValue(child, "name", parameter.getKey());
				PluginUtils.setValue(child, "value", parameter.getValue());
				element.addContent(child);
			}
		}
	}
}
