// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate;

import java.awt.*;
import static java.awt.GridBagConstraints.*;
import java.util.Set;
import javax.swing.*;
import javax.swing.text.JTextComponent;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.DialogWrapper;
import com.intellij.openapi.help.HelpManager;

import com.kiwisoft.generate.model.*;
import com.kiwisoft.utils.ObservableDynaBean;
import com.kiwisoft.utils.ListenerSupport;
import com.kiwisoft.utils.gui.DecoratedTextField;
import com.kiwisoft.utils.gui.DecoratedComponent;
import com.kiwisoft.utils.gui.DecoratedFormattedTextField;
import com.kiwisoft.utils.gui.lookup.LookupField;
import com.kiwisoft.utils.gui.lookup.TextLookup;

/**
 * Dialog for entering the template parameters.
 *
 * @author Stefan Stiller
 */
public class ParameterDialog extends DialogWrapper
{
	private Template template;
	private ParameterBean bean;
	private ListenerSupport listenerSupport=new ListenerSupport();

	public ParameterDialog(Project project, Template template)
	{
		super(project, true);
		setTitle("Enter Template Parameters");
		this.template=template;
		try
		{
			this.bean=(ParameterBean)template.createDynaClass().newInstance();
			init();
		}
		catch (Exception e)
		{
			throw new RuntimeException(e);
		}
	}

	protected void dispose()
	{
		listenerSupport.dispose();
		super.dispose();
	}

	protected Action[] createActions()
	{
		return new Action[]{getOKAction(), getCancelAction(), getHelpAction()};
	}

	protected JComponent createCenterPanel()
	{
		JPanel panel=new JPanel();
		initializeParameterGroupPanel(panel, bean, template, listenerSupport);
		JScrollPane scrollPane=new JScrollPane(panel);
		scrollPane.setPreferredSize(new Dimension(500, 300));
		return scrollPane;
	}

	static void initializeParameterGroupPanel(JPanel panel, ParameterBean bean, UserType userType, ListenerSupport listenerSupport)
	{
		int row=0;
		panel.setLayout(new GridBagLayout());
		for (UserParameter parameter : userType.getParameters())
		{
			row++;
			if (parameter.getType() instanceof UserType)
			{
				if (parameter.isCollection())
				{
					GroupListPanel parameterPanel=new GroupListPanel(bean, parameter);
					listenerSupport.addDisposable(parameterPanel);
					panel.add(parameterPanel, new GridBagConstraints(0, row, 2, 1, 1.0, 0.0, CENTER, HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
				}
				else
				{
					OptionalGroupPanel parameterPanel=new OptionalGroupPanel(bean, parameter);
					listenerSupport.addDisposable(parameterPanel);
					panel.add(parameterPanel, new GridBagConstraints(0, row, 2, 1, 1.0, 0.0, CENTER, HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
				}
			}
			else
			{
				if (parameter.isCollection())
				{
					panel.add(new PrimitiveListPanel(bean, parameter),
							  new GridBagConstraints(0, row, 2, 1, 1.0, 0.0, CENTER, HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
				}
				else
				{
					addSimpleParameter(panel, row, bean, parameter, listenerSupport);
				}
			}
		}
		row++;
		panel.add(Box.createGlue(),
				  new GridBagConstraints(0, row, 1, 1, 0.0, 1.0, CENTER, NONE, new Insets(0, 0, 0, 0), 0, 0));
	}

	private static void addSimpleParameter(JPanel panel, int row, ParameterBean bean, UserParameter parameter, ListenerSupport listenerSupport)
	{
		JLabel label=new JLabel(parameter.getDisplayableName()+":");
		label.setToolTipText(parameter.getDescription());
		if (parameter.isRequired())
			label.setFont(label.getFont().deriveFont(Font.BOLD));

		panel.add(label,
				  new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, WEST, NONE, new Insets(5, 5, 5, 5), 0, 0));
		JComponent component=null;
		Type type=parameter.getType();
		if (type instanceof StringType)
		{
			Set values=parameter.getValues();
			if (values!=null && !values.isEmpty())
			{
				//noinspection unchecked
				component=new LookupField(new DecoratedTextField(30), new TextLookup(values, parameter.isRestricted()));
				panel.add(component, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
				listenerSupport.bind((LookupField)component, bean, parameter.getName());
			}
			else
			{
				component=new DecoratedTextField(30);
				panel.add(component, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
				listenerSupport.bind((JTextComponent)component, bean, parameter.getName());
			}
		}
		else if (type instanceof BooleanType)
		{
			component=new JCheckBox();
			panel.add(component, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
			listenerSupport.bind((JCheckBox)component, bean, parameter.getName());
		}
		else if (type instanceof IntegerType || type instanceof DecimalType)
		{
			component=new DecoratedFormattedTextField();
			((JFormattedTextField)component).setColumns(8);
			panel.add(component, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, WEST, NONE, new Insets(5, 0, 5, 5), 0, 0));
			listenerSupport.bindNumber((JFormattedTextField)component, bean, parameter.getName());
		}
		if (component!=null)
		{
			if (component instanceof DecoratedComponent && parameter.isRequired())
				((DecoratedComponent)component).setDecorators(new RequiredDecorator());
			component.setToolTipText(parameter.getDescription());
			listenerSupport.installFocusListener(component, new ComponentAligner(panel, component));
		}
	}

	public ObservableDynaBean getBean()
	{
		return bean;
	}

	@Override
	public void doOKAction()
	{
		if (bean.isValid())
		{
			super.doOKAction();
		}
		else
		{
			JOptionPane.showMessageDialog(getContentPane(), "Missing required data.", "Validation Error", JOptionPane.ERROR_MESSAGE);
		}
	}

	protected void doHelpAction()
	{
		HelpManager.getInstance().invokeHelp("KiwiGenerator.parameters");
	}
}
