// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model;

import static java.util.Collections.unmodifiableSet;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.Map;
import java.util.Set;
import java.util.HashSet;

import com.kiwisoft.utils.*;
import com.kiwisoft.utils.xml.*;

/**
 * A user defined template parameter. 
 *
 * @author Stefan Stiller
 */
public class UserParameter extends XMLAdapter implements Parameter, ObservableBean
{
	private String name;
	private String displayName;
	private String description;
	private Type type=Types.valueOf("String");
	private String defaultValue;
	private boolean required;
	private boolean collection;
	private boolean restricted;
	private Set values;

	private PropertyChangeSupport changeSupport=new PropertyChangeSupport(this);
	private CollectionChangeSupport collectionSupport=new CollectionChangeSupport(this);

	public UserParameter()
	{
	}

	public String getName()
	{
		return name;
	}

	public void setName(String name)
	{
		String oldName=this.name;
		this.name=name;
		firePropertyChange("name", oldName, name);
	}

	public String getDisplayableName()
	{
		if (!StringUtils.isEmpty(displayName)) return displayName;
		return name;
	}

	public String getDisplayName()
	{
		return displayName;
	}

	public void setDisplayName(String displayName)
	{
		String oldDisplayName=this.displayName;
		this.displayName=displayName;
		firePropertyChange("displayName", oldDisplayName, displayName);
	}

	public String getDescription()
	{
		return description;
	}

	public void setDescription(String description)
	{
		String oldDescription=this.description;
		this.description=description;
		firePropertyChange("description", oldDescription, description);
	}

	public Type getType()
	{
		return type;
	}

	public void setType(Type type)
	{
		Type oldType=this.type;
		this.type=type;
		if (type!=null && oldType!=null)
		{
			if (values!=null)
			{
				Set newValues=new HashSet(values.size());
				for (Object value : values)
				{
					Object newValue=type.parse(oldType.format(value));
					if (newValue!=null)
					{
						//noinspection unchecked
						newValues.add(newValue);
					}
				}
				setValues(newValues);
			}
		}
		else setValues(null);
		firePropertyChange("type", oldType, type);
	}

	public boolean isRequired()
	{
		return required;
	}

	public void setRequired(boolean required)
	{
		boolean oldRequired=this.required;
		this.required=required;
		firePropertyChange("required", oldRequired, required);
	}

	public boolean isCollection()
	{
		return collection;
	}

	public void setCollection(boolean collection)
	{
		boolean oldCollection=this.collection;
		this.collection=collection;
		firePropertyChange("collection", oldCollection, collection);
	}

	public boolean isRestricted()
	{
		return restricted;
	}

	public void setRestricted(boolean restricted)
	{
		boolean oldRestricted=this.restricted;
		this.restricted=restricted;
		firePropertyChange("restricted", oldRestricted, restricted);
	}

	public Object getDefaultValue()
	{
		if (type!=null && defaultValue!=null) return type.parse(defaultValue);
		return null;
	}

	public void setDefaultValue(Object defaultValue)
	{
		Object oldValue=getDefaultValue();
		String stringValue=null;
		if (type!=null) stringValue=type.format(defaultValue);
		this.defaultValue=stringValue;
		firePropertyChange("defaultValue", oldValue, defaultValue);
	}

	public Set getValues()
	{
		//noinspection unchecked
		return values!=null ? unmodifiableSet(values) : null;
	}

	public void setValues(Set values)
	{
		Set oldValues=this.values;
		//noinspection unchecked
		this.values=values!=null ? new HashSet(values) : null;
		firePropertyChange("values", oldValues, getValues());
	}

	// PropertyChangeSupport delegates

	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}

	public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(propertyName, listener);
	}

	public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(propertyName, listener);
	}

	public void firePropertyChange(String propertyName, Object oldValue, Object newValue)
	{
		changeSupport.firePropertyChange(propertyName, oldValue, newValue);
	}

	// CollectionChangeSupport delegates

	public void addListener(CollectionChangeListener listener)
	{
		collectionSupport.addListener(listener);
	}

	public void removeListener(CollectionChangeListener listener)
	{
		collectionSupport.removeListener(listener);
	}

	public void fireElementAdded(String propertyName, Object newElement)
	{
		collectionSupport.fireElementAdded(propertyName, newElement);
	}

	public void fireElementRemoved(String propertyName, Object oldElement)
	{
		collectionSupport.fireElementRemoved(propertyName, oldElement);
	}

	// XMLObject interface methods

	public UserParameter(XMLContext context, String name)
	{
		super(context, name);
	}

	public void setXMLAttribute(XMLContext context, String name, String value)
	{
		if ("name".equalsIgnoreCase(name)) this.name=value;
		else if ("displayName".equalsIgnoreCase(name)) this.displayName=value;
		else if ("description".equalsIgnoreCase(name)) this.description=value;
		else if ("required".equalsIgnoreCase(name)) this.required=Boolean.parseBoolean(value);
		else if ("collection".equalsIgnoreCase(name)) this.collection=Boolean.parseBoolean(value);
		else if ("default".equalsIgnoreCase(name)) this.defaultValue=value;
		else if ("restricted".equalsIgnoreCase(name)) this.restricted=Boolean.parseBoolean(value);
	}

	public void setXMLReference(XMLContext context, String name, Object value)
	{
		if ("type".equalsIgnoreCase(name)) setType((Type)value);
	}

	public void addXMLElement(XMLContext context, XMLObject element)
	{
		if (element instanceof DefaultXMLObject)
		{
			DefaultXMLObject xmlObject=(DefaultXMLObject)element;
			if (type!=null && "value".equalsIgnoreCase(xmlObject.getName()))
			{
				String stringValue=xmlObject.getContent();
				if (stringValue!=null)
				{
					Object value=type.parse(stringValue);
					if (value!=null)
					{
						if (values==null) values=new HashSet();
						//noinspection unchecked
						values.add(value);
					}
				}
			}
		}
		super.addXMLElement(context, element);
	}

	public void writeXML(XMLWriter xmlWriter) throws IOException
	{
		xmlWriter.startElement("parameter");
		xmlWriter.setAttribute("name", name);
		xmlWriter.setAttribute("displayName", displayName);
		xmlWriter.setAttribute("description", description);
		xmlWriter.setAttribute("required", required);
		xmlWriter.setAttribute("collection", collection);
		if (type!=null) xmlWriter.setAttribute("type", type.getName());
		xmlWriter.setAttribute("default", defaultValue);
		xmlWriter.setAttribute("restricted", restricted);
		if (values!=null && type!=null)
		{
			for (Object value : values)
			{
				xmlWriter.addElement("value", type.format(value));
			}
		}
		xmlWriter.closeElement("parameter");
	}

	public UserParameter copy(Map<UserType, UserType> typeMap)
	{
		UserParameter parameter=new UserParameter();
		parameter.setName(getName());
		parameter.setDisplayName(getDisplayName());
		parameter.setDescription(getDescription());
		Type type=getType();
		if (type instanceof UserType) type=typeMap.get(type);
		parameter.setType(type);
		parameter.setValues(getValues());
		parameter.setRequired(isRequired());
		parameter.setCollection(isCollection());
		parameter.setRestricted(isRestricted());
		parameter.setDefaultValue(getDefaultValue());
		return parameter;
	}

	public void validate()
	{
		if (StringUtils.isEmpty(getName())) throw new RuntimeException("Missing parameter name.");
		if (!Utils.isJavaIdentifier(getName()))
			throw new RuntimeException("Name of parameter '"+getName()+"' is no Java identifier.");
		if (type==null) throw new RuntimeException("Missing type of parameter '"+getName()+"'.");
		if (restricted && (values==null || values.isEmpty()))
			throw new RuntimeException("No values defined for restricted parameter.");
	}
}
