// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model.ui;

import java.awt.*;
import static java.awt.GridBagConstraints.*;
import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;

import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.ui.DialogWrapper;
import com.intellij.openapi.help.HelpManager;
import com.intellij.util.ui.tree.TreeUtil;

import com.kiwisoft.generate.model.*;
import com.kiwisoft.generate.tree.*;
import com.kiwisoft.utils.ListenerSupport;
import com.kiwisoft.utils.Utils;
import com.kiwisoft.utils.gui.GuiUtils;
import com.kiwisoft.utils.gui.Icons;
import com.kiwisoft.utils.gui.tree.GenericTree;
import com.kiwisoft.utils.gui.tree.GenericTreeNode;

/**
 * A dialog for editing a template.
 *
 * @author Stefan Stiller
 */
public class TemplateDialog extends DialogWrapper implements TreeSelectionListener
{
	private Template template;
	private GenericTree tree;
	private JPanel rightContainerPanel;
	private DetailsPanel detailsPanel;
	private ListenerSupport listenerSupport=new ListenerSupport();

	public TemplateDialog(JComponent parent, Template template)
	{
		super(parent, true);
		setTitle("Define Template");
		this.template=template;
		init();
	}

	protected Action[] createActions()
	{
		return new Action[]{getOKAction(), getCancelAction(), getHelpAction()};
	}

	protected JComponent createCenterPanel()
	{
		JTextField nameField=new JTextField();
		JTextField descriptionField=new JTextField();
		JCheckBox sourceFolderField=new JCheckBox("Requires source folder");

		JPanel panel=new JPanel(new GridBagLayout());
		panel.setBorder(BorderFactory.createEtchedBorder());
		panel.add(new JLabel("Name:"), new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, WEST, NONE, new Insets(5, 5, 5, 5), 0, 0));
		panel.add(nameField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
		panel.add(new JLabel("Description:"), new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, WEST, NONE, new Insets(5, 5, 5, 5), 0, 0));
		panel.add(descriptionField, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
		panel.add(sourceFolderField, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0, WEST, NONE, new Insets(5, 0, 5, 5), 0, 0));

		listenerSupport.bind(nameField, template, "name");
		listenerSupport.bind(descriptionField, template, "description");
		listenerSupport.bind(sourceFolderField, template, "requiresSourceFolder");

		JPanel centerPanel=new JPanel(new BorderLayout(5, 5));
		centerPanel.add(panel, BorderLayout.NORTH);
		centerPanel.add(createParametersPanel(), BorderLayout.CENTER);
		return centerPanel;
	}

	private JComponent createParametersPanel()
	{
		TemplateNode templateNode=new TemplateNode(template);
		tree=new GenericTree(templateNode);
		listenerSupport.addDisposable(tree);
		listenerSupport.installSelectionListener(tree, this);
		tree.setShowsRootHandles(true);
		tree.setRootVisible(false);
		TreeUtil.expandAll(tree);
		JScrollPane treePane=new JScrollPane(tree);
		treePane.setPreferredSize(new Dimension(200, 400));

		rightContainerPanel=new JPanel(new BorderLayout());
		rightContainerPanel.setPreferredSize(new Dimension(400, 400));

		JSplitPane splitPane=new JSplitPane();
		splitPane.setLeftComponent(treePane);
		splitPane.setRightComponent(rightContainerPanel);

		JPanel panel=new JPanel(new BorderLayout());
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(), BorderFactory.createEmptyBorder(5, 5, 5, 5)));
		panel.add(createToolbar(), BorderLayout.NORTH);
		panel.add(splitPane, BorderLayout.CENTER);
		return panel;
	}

	private JComponent createToolbar()
	{
		DefaultActionGroup actionGroup=new DefaultActionGroup("KiwiCreate.TemplateDialog", false);
		actionGroup.add(new AddAction());
		actionGroup.add(new RemoveAction());
		actionGroup.addSeparator();
		actionGroup.add(new MoveUpAction());
		actionGroup.add(new MoveDownAction());
		ActionManager actionManager=ActionManager.getInstance();
		return actionManager.createActionToolbar("KiwiCreate.TemplateDialog", actionGroup, true).getComponent();
	}

	public void valueChanged(TreeSelectionEvent e)
	{
		Object oldValue=detailsPanel!=null ? detailsPanel.getObject() : null;
		TreePath selectionPath=e.getNewLeadSelectionPath();
		Object newValue=null;
		if (selectionPath!=null)
		{
			GenericTreeNode treeNode=(GenericTreeNode)selectionPath.getLastPathComponent();
			if (treeNode!=null) newValue=treeNode.getUserObject();
		}
		if (!Utils.equals(oldValue, newValue))
		{
			if (detailsPanel==null || detailsPanel.stopEditing())
			{
				rightContainerPanel.removeAll();
				if (detailsPanel!=null) detailsPanel.dispose();
				detailsPanel=null;
				if (selectionPath!=null)
				{
					GenericTreeNode treeNode=(GenericTreeNode)selectionPath.getLastPathComponent();
					if (treeNode!=null)
					{
						Object object=treeNode.getUserObject();
						if (object instanceof UserParameter)
							detailsPanel=new UserParameterPanel(template, (UserParameter)object);
						else if (object instanceof SystemParameter)
							detailsPanel=new SystemParameterPanel((SystemParameter)object);
						else if (object instanceof SystemMethod)
							detailsPanel=new SystemMethodPanel((SystemMethod)object);
						else if (object instanceof UserType)
							detailsPanel=new UserTypePanel((UserType)object);
						else if (object instanceof TemplateFile)
							detailsPanel=new TemplateFilePanel((TemplateFile)object);
					}
				}
				if (detailsPanel!=null)
				{
					rightContainerPanel.add(detailsPanel, BorderLayout.CENTER);
				}
				rightContainerPanel.updateUI();
			}
			else
			{
				TreePath oldLeadSelectionPath=e.getOldLeadSelectionPath();
				tree.setSelectionPath(oldLeadSelectionPath);
			}
		}
	}

	protected void dispose()
	{
		if (detailsPanel!=null) detailsPanel.dispose();
		detailsPanel=null;
		listenerSupport.dispose();
		super.dispose();
	}

	private class AddAction extends AnAction
	{
		private AddAction()
		{
			super("Add", "Add a new element.", Icons.getIcon("add"));
		}

		public void update(AnActionEvent e)
		{
			super.update(e);
			GenericTreeNode treeNode=tree.getSelectedNode();
			e.getPresentation().setEnabled(treeNode instanceof ParameterNode
										   || treeNode instanceof UserParametersNode
										   || treeNode instanceof UserTypesNode
										   || treeNode instanceof UserTypeNode
										   || treeNode instanceof FilesNode
										   || treeNode instanceof TemplateFileNode);
		}

		public void actionPerformed(AnActionEvent e)
		{
			GenericTreeNode treeNode=tree.getSelectedNode();
			if (treeNode instanceof ParameterNode || treeNode instanceof TemplateFileNode) treeNode=treeNode.getParent();
			if (treeNode instanceof UserParametersNode)
			{
				UserParameter newParameter=new UserParameter();
				template.addParameter(newParameter);
				selectTreeObject(treeNode, newParameter);
			}
			else if (treeNode instanceof UserTypeNode)
			{
				UserParameter newParameter=new UserParameter();
				//noinspection unchecked
				((UserTypeNode)treeNode).getUserObject().addParameter(newParameter);
				selectTreeObject(treeNode, newParameter);
			}
			else if (treeNode instanceof UserTypesNode)
			{
				UserType newType=new UserType();
				template.addType(newType);
				selectTreeObject(treeNode, newType);
			}
			else if (treeNode instanceof FilesNode)
			{
				TemplateFile newFile=new TemplateFile();
				template.addFile(newFile);
				selectTreeObject(treeNode, newFile);
			}
		}

	}

	private void selectTreeObject(final GenericTreeNode treeNode, final Object object)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				GenericTreeNode childNode=treeNode.getChildNode(object);
				if (childNode!=null) TreeUtil.selectNode(tree, childNode);
			}
		});
	}

	private class RemoveAction extends AnAction
	{
		private RemoveAction()
		{
			super("Remove", "Remove the selected element from the template.", Icons.getIcon("remove"));
		}

		public void update(AnActionEvent e)
		{
			super.update(e);
			GenericTreeNode selectedNode=tree.getSelectedNode();
			e.getPresentation().setEnabled(selectedNode instanceof ParameterNode
										   || selectedNode instanceof UserTypeNode
										   || selectedNode instanceof TemplateFileNode);
		}

		@SuppressWarnings({"unchecked"})
		public void actionPerformed(AnActionEvent e)
		{
			GenericTreeNode selectedNode=tree.getSelectedNode();
			if (selectedNode instanceof UserTypeNode)
				template.removeType(((UserTypeNode)selectedNode).getUserObject());
			else if (selectedNode instanceof TemplateFileNode)
				template.removeFile(((TemplateFileNode)selectedNode).getUserObject());
			else if (selectedNode instanceof ParameterNode)
			{
				GenericTreeNode parentNode=selectedNode.getParent();
				UserParameter parameter=((ParameterNode)selectedNode).getUserObject();
				if (parentNode instanceof UserTypeNode)
					((UserTypeNode)parentNode).getUserObject().removeParameter(parameter);
				else if (parentNode instanceof UserParametersNode)
					template.removeParameter(parameter);
			}
		}
	}

	private class MoveUpAction extends AnAction
	{
		public MoveUpAction()
		{
			super("Move Up", "Move the selected element up.", Icons.getIcon("move.up"));
		}

		public void update(AnActionEvent e)
		{
			super.update(e);
			GenericTreeNode selectedNode=tree.getSelectedNode();
			e.getPresentation().setEnabled(selectedNode instanceof ParameterNode && selectedNode.canMoveUp());
		}

		public void actionPerformed(AnActionEvent e)
		{
			final GenericTreeNode selectedNode=tree.getSelectedNode();
			if (selectedNode instanceof ParameterNode)
			{
				selectedNode.moveUp();
				SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						tree.setSelectionPath(GuiUtils.getPath(selectedNode));
					}
				});
			}
		}
	}

	private class MoveDownAction extends AnAction
	{
		public MoveDownAction()
		{
			super("Move Down", "Move the selected element down.", Icons.getIcon("move.down"));
		}

		public void update(AnActionEvent e)
		{
			super.update(e);
			GenericTreeNode selectedNode=tree.getSelectedNode();
			e.getPresentation().setEnabled(selectedNode instanceof ParameterNode && selectedNode.canMoveDown());
		}

		public void actionPerformed(AnActionEvent e)
		{
			final GenericTreeNode selectedNode=tree.getSelectedNode();
			if (selectedNode instanceof ParameterNode)
			{
				selectedNode.moveDown();
				SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						tree.setSelectionPath(GuiUtils.getPath(selectedNode));
					}
				});
			}
		}
	}

	protected void doOKAction()
	{
		try
		{
			template.validate();
			super.doOKAction();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(getContentPane(), e.getMessage(), "Invalid Template", JOptionPane.ERROR_MESSAGE);
		}
	}

	protected void doHelpAction()
	{
		String topic=null;
		if (detailsPanel!=null) topic=detailsPanel.getHelpTopic();
		if (topic==null) topic="KiwiGenerator.template.details";
		HelpManager.getInstance().invokeHelp(topic);
	}
}
