// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils;

import java.util.HashMap;
import java.util.Map;

/**
 * A class for managing various features connected to classes or interfaces. The class will automatically
 * look for features defined for parent classes and interfaces if there is no feature defined for a class. 
 *
 * @author Stefan Stiller
 */
public class ClassFeatureManager<T>
{
	public static final String DEFAULT="Default";

	private Map<Class, T> defaultFeatures=new HashMap<Class, T>();
	private DoubleKeyMap<Class, String, T> registeredFeatures=new DoubleKeyMap<Class, String, T>();
	private DoubleKeyMap<Class, String, T> features=new DoubleKeyMap<Class, String, T>();

	protected ClassFeatureManager()
	{
	}

	protected T getFeature(Class aClass)
	{
		return getFeature(aClass, null);
	}

	protected T getRegisteredFeature(Class aClass)
	{
		return getRegisteredFeature(aClass, null);
	}

	protected T getFeature(Class aClass, String variant)
	{
		aClass=Utils.convertPrimitiveClass(aClass);
		if (variant==null || DEFAULT.equals(variant))
		{
			T feature=getDefaultFeature(aClass);
			defaultFeatures.put(aClass, feature);
			return feature;
		}
		else
		{
			T feature=getVariantFeature(aClass, variant);
			if (feature==null) feature=getFeature(aClass, null);
			features.put(aClass, variant, feature);
			return feature;
		}
	}

	protected T getRegisteredFeature(Class aClass, String variant)
	{
		aClass=Utils.convertPrimitiveClass(aClass);
		if (variant==null || DEFAULT.equals(variant)) return defaultFeatures.get(aClass);
		else return registeredFeatures.get(aClass, variant);
	}

	protected void setFeature(Class aClass, T feature)
	{
		setFeature(aClass, null, feature);
	}

	protected void setFeature(Class aClass, String variant, T feature)
	{
		aClass=Utils.convertPrimitiveClass(aClass);
		if (variant==null || DEFAULT.equals(variant))
		{
			defaultFeatures.put(aClass, feature);
		}
		else
		{
			registeredFeatures.put(aClass, variant, feature);
			features.remove(aClass, variant);
		}
	}

	protected T getDefaultFeature(Class aClass)
	{
		if (aClass==null) return null;
		T feature=defaultFeatures.get(aClass);
		if (feature==null) feature=getInheritedDefaultFeature(aClass);
		return feature;
	}

	protected T getInheritedDefaultFeature(Class aClass)
	{
		Class[] interfaces=aClass.getInterfaces();
		T feature=null;
		for (int i=0; i<interfaces.length && feature==null; i++)
		{
			Class anInterface=interfaces[i];
			feature=getDefaultFeature(anInterface);
		}
		if (feature==null) feature=getDefaultFeature(aClass.getSuperclass());
		return feature;
	}

	protected T getVariantFeature(Class aClass, String variant)
	{
		T feature=registeredFeatures.get(aClass, variant);
		if (feature==null) feature=features.get(aClass, variant);
		if (feature==null) feature=getInheritedVariantFeature(aClass, variant);
		return feature;
	}

	protected T getInheritedVariantFeature(Class aClass, String variant)
	{
		if (aClass.getSuperclass()!=null) return getVariantFeature(aClass.getSuperclass(), variant);
		return null;
	}
}
