// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.table;

import java.util.*;

import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.AbstractTableModel;

import com.kiwisoft.utils.gui.ObjectStyle;
import com.kiwisoft.utils.Utils;

/**
 * {@link javax.swing.table.TableModel} implementation which is based on sortable row
 * objects.
 *
 * @author Stefan Stiller
 */
public abstract class SortableTableModel<T> extends AbstractTableModel implements TableRowListener, TableConstants
{
	/**
	 * List of sort column descriptions.
	 */
	private List<TableSortDescription> sortColumns=new ArrayList<TableSortDescription>(1);

	/**
	 * The list of all table rows.
	 */
	protected List<SortableTableRow<T>> rows=new ArrayList<SortableTableRow<T>>();

	/**
	 * The comparator to sort the table rows.
	 */
	protected Comparator<SortableTableRow> comparator=new DefaultComparator();

	/**
	 * Adds a new row to the end of the table.
	 *
	 * @param row The new row.
	 */
	public void addRow(SortableTableRow<T> row)
	{
		rows.add(row);
		row.addRowListener(this);
		row.installListener();
		int index=rows.size()-1;
		fireTableRowsInserted(index, index);
	}

	/**
	 * Adds a new row at the specified index of the table.
	 *
	 * @param row The new row.
	 * @param index The index at which the row should be inserted.
	 */
	public void addRow(SortableTableRow<T> row, int index)
	{
		rows.add(index, row);
		row.addRowListener(this);
		row.installListener();
		index=rows.indexOf(row);
		fireTableRowsInserted(index, index);
	}

	/**
	 * Removes the specified row from the table.
	 *
	 * @param row The row to be removed.
	 */
	public void removeRow(SortableTableRow<T> row)
	{
		int index=rows.indexOf(row);
		if (index>=0)
		{
			row.removeListener();
			row.removeRowListener(this);
			rows.remove(row);
			fireTableRowsDeleted(index, index);
		}
	}

	/**
	 * Removes the row at the specified index from the table.
	 *
	 * @param index The index to be removed.
	 */
	public void removeRow(int index)
	{
		SortableTableRow row=rows.get(index);
		row.removeListener();
		row.removeRowListener(this);
		rows.remove(index);
		fireTableRowsDeleted(index, index);
	}

	/**
	 * Remove all rows there the user object is equal to the specified object.
	 */
	public void removeRow(T userObject)
	{
		for (SortableTableRow<T> row : new ArrayList<SortableTableRow<T>>(rows))
		{
			if (Utils.equals(row.getUserObject(), userObject)) removeRow(row);
		}
	}

	/**
	 * Checks if the table model contains a row which references the specified
	 * user object.
	 */
	public boolean containsObject(Object userObject)
	{
		for (SortableTableRow row : new LinkedList<SortableTableRow>(rows))
		{
			if (Utils.equals(row.getUserObject(), userObject)) return true;
		}
		return false;
	}

	/**
	 * Returns the index of the first row which references the specified user
	 * object.
	 */
	public int indexOf(Object userObject)
	{
		for (int i=0; i<rows.size(); i++)
		{
			SortableTableRow row=rows.get(i);
			if (Utils.equals(row.getUserObject(), userObject)) return i;
		}
		return -1;
	}

	public int indexOfRow(SortableTableRow row)
	{
		for (int i=0; i<rows.size(); i++)
		{
			if (row==rows.get(i)) return i;
		}
		return -1;
	}

	public void setRows(List<SortableTableRow<T>> rows)
	{
		for (SortableTableRow<T> row : this.rows)
		{
			row.removeListener();
			row.removeRowListener(this);
		}
		this.rows.clear();
		for (SortableTableRow<T> row : rows)
		{
			row.addRowListener(this);
			row.installListener();
			this.rows.add(row);
		}
		sort();
	}

	public void clear()
	{
		for (SortableTableRow row : rows)
		{
			row.removeListener();
			row.removeRowListener(this);
		}
		rows.clear();
		fireTableDataChanged();
	}

	public T getObject(int row)
	{
		return getRow(row).getUserObject();
	}

	public SortableTableRow<T> getRow(int row)
	{
		if (row>=0 && row<rows.size()) return rows.get(row);
		else return null;
	}

	public int getRowCount()
	{
		return rows.size();
	}

	public Class getCellClass(int row, int col)
	{
		Class cellClass=getRow(row).getCellClass(col);
		if (cellClass!=null) return cellClass;
		else
		{
			Object value=getValueAt(row, col);
			if (value!=null) return value.getClass();
			else return Object.class;
		}
	}

	public TableCellRenderer getCellRenderer(int row, int col)
	{
		Class cellClass=getCellClass(row, col);
		if (cellClass==null) cellClass=Object.class;
		String cellFormat=getCellFormat(row, col);
		ObjectStyle cellStyle=getCellStyle(row, col);
		return TableRendererFactory.getInstance().getRenderer(cellClass, cellFormat, cellStyle);
	}

	public TableCellEditor getCellEditor(int row, int col)
	{
		ObjectEditor editor=getObjectEditor(row, col);
		ObjectStyle cellStyle=getCellStyle(row, col);
		if (editor!=null) return TableEditorFactory.getInstance().wrapEditor(editor, cellStyle, false);
		Class cellClass=getCellClass(row, col);
		if (cellClass==null) cellClass=Object.class;
		String cellFormat=getCellFormat(row, col);
		return TableEditorFactory.getInstance().getEditor(cellClass, cellFormat, cellStyle);
	}

	/**
	 * Checks if this table cell is editable. This implementation forwards the
	 * request to the table row.
	 *
	 * @param row The row being queried.
	 * @param column The column being queried.
	 * @return <code>true</code> if the cell is editable.
	 */
	public boolean isCellEditable(int row, int column)
	{
		return getRow(row).isEditable(column);
	}

	public String getCellFormat(int row, int col)
	{
		return getRow(row).getCellFormat(col);
	}

	public ObjectEditor getObjectEditor(int row, int col)
	{
		return getRow(row).getObjectEditor(col);
	}

	public ObjectStyle getCellStyle(int row, int col)
	{
		return getRow(row).getCellStyle(col);
	}

	public Object getToolTipText(int row, int col)
	{
		return getRow(row).getToolTipText(col);
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
		switch (getRow(rowIndex).setValue(aValue, columnIndex))
		{
			case CELL_UPDATE:
				fireTableCellUpdated(rowIndex, columnIndex);
				break;
			case ROW_UPDATE:
				fireTableRowsUpdated(rowIndex, rowIndex);
				break;
			case TABLE_UPDATE:
				fireTableDataChanged();
		}
	}

	public Object getValueAt(int rowIndex, int columnIndex)
	{
		return getRow(rowIndex).getDisplayValue(columnIndex);
	}

	public TableSortDescription getSortDescription(int columnIndex)
	{
		for (Iterator it=sortColumns.iterator(); it.hasNext();)
		{
			TableSortDescription sortDescription=(TableSortDescription)it.next();
			if (sortDescription.getColumn()==columnIndex) return sortDescription;
		}
		return null;
	}

	public int getSortIndex(int columnIndex)
	{
		for (int i=0; i<sortColumns.size(); i++)
		{
			TableSortDescription sortDescription=sortColumns.get(i);
			if (sortDescription.getColumn()==columnIndex) return i;
		}
		return -1;
	}

	public void addSortColumn(TableSortDescription sortDescription)
	{
		sortColumns.add(sortDescription);
	}

	public void addSortColumn(int columnIndex, boolean add)
	{
		TableSortDescription sortDescription=getSortDescription(columnIndex);
		boolean singleSort=sortColumns.size()==1;
		if (!add) sortColumns.clear();
		if (sortDescription==null || (!add && !singleSort)) sortColumns.add(new TableSortDescription(columnIndex, ASCEND));
		else if (ASCEND.equals(sortDescription.getDirection()))
		{
			sortDescription.setDirection(DESCEND);
			sortColumns.remove(sortDescription);
			sortColumns.add(sortDescription);
		}
		else if (DESCEND.equals(sortDescription.getDirection())) sortColumns.remove(sortDescription);
		sort();
	}

	public void clearSort()
	{
		sortColumns.clear();
	}

	public void sort()
	{
		if (!sortColumns.isEmpty()) Collections.sort(rows, comparator);
		fireTableDataChanged();
	}

	public void rowUpdated(SortableTableRow row)
	{
		int index=rows.indexOf(row);
		if (index>=0) fireTableRowsUpdated(index, index);
	}

	public void rowDeleted(SortableTableRow row)
	{
		int index=rows.indexOf(row);
		if (index>=0) removeRow(index);
	}

	public boolean isResortable()
	{
		return true;
	}

	private class DefaultComparator implements Comparator<SortableTableRow>
	{
		public int compare(SortableTableRow row1, SortableTableRow row2)
		{
			if (row1.getSortPriority()==row2.getSortPriority())
			{
				for (Iterator it=sortColumns.iterator(); it.hasNext();)
				{
					TableSortDescription sortDescription=(TableSortDescription)it.next();
					int column=sortDescription.getColumn();
					Comparable value1=row1.getSortValue(column);
					Comparable value2=row2.getSortValue(column);
					int compareResult;
					int direction=sortDescription.getDirection().intValue();
					if (value1.getClass()==value2.getClass())
					{
						//noinspection unchecked
						compareResult=direction*value1.compareTo(value2);
					}
					else compareResult=direction*value1.getClass().getName().compareTo(value2.getClass().getName());
					if (compareResult!=0) return compareResult;
				}
				return 0;
			}
			else if (row1.getSortPriority()<row2.getSortPriority()) return -1;
			else return 1;
		}
	}

	public boolean removeSelectedRows(JTable table)
	{
		int[] rows=table.getSelectedRows();
		if (rows!=null && rows.length>0)
		{
			final Set<SortableTableRow<T>> tableRows=new HashSet<SortableTableRow<T>>();
			for (int i=0; i<rows.length; i++)
			{
				tableRows.add(getRow(rows[i]));
			}
			SwingUtilities.invokeLater(new Runnable()
			{
				public void run()
				{
					for (Iterator<SortableTableRow<T>> it=tableRows.iterator(); it.hasNext();)
					{
						removeRow(it.next());
					}
				}
			});
			return true;
		}
		return false;
	}
}
